#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

import decimaux
import operations
import droites
import quotients
import fractions
import angles
import symetrie
from random import randrange
from outils import tex_entete

import gettext
import locale
locale.setlocale(locale.LC_ALL, "")
locale.textdomain("pyromaths")
gettext.textdomain("pyromaths")
_=locale.gettext


def tex_ecrit_nombre(f0, f1):
    f0.write("""
\\exercice
""")
    f0.write('\\begin{enumerate}\n')
    f0.write(_('  \\item \\\'Ecrire en chiffres les nombres suivants.\n'))
    f0.write('    \\begin{enumerate}\n')
    f1.write("""
\\exercice*
""")
    f1.write('\\begin{enumerate}\n')
    f1.write(_('  \\item \\\'Ecrire en chiffres les nombres suivants.\n'))
    f1.write('    \\begin{enumerate}\n')
    decimaux.EcritEnChiffre(f0, f1)
    f0.write('    \\end{enumerate}\n')
    f0.write(_('  \\item \\\'Ecrire en lettres les nombres suivants (sans utiliser le mot ``virgule").\n'))
    f0.write('    \\begin{enumerate}\n')
    f1.write('    \\end{enumerate}\n')
    f1.write(_('  \\item \\\'Ecrire en lettres les nombres suivants (sans utiliser le mot ``virgule").\n'))
    f1.write('    \\begin{enumerate}\n')
    decimaux.EcritEnLettre(f0, f1)
    f0.write('    \\end{enumerate}\n')
    f0.write('\\end{enumerate}\n')
    f1.write('    \\end{enumerate}\n')
    f1.write('\\end{enumerate}\n')


def tex_operations(f0, f1):
    nb_exos = 3
    tex_exos = (operations.tex_somme, operations.tex_difference,
                operations.tex_produit)
    ordre_exos = [i for i in xrange(nb_exos)]
    f0.write("""
\\exercice
""")
    f0.write(_('Poser et effectuer les operations suivantes.\n'))
    f0.write('\\begin{multicols}{2}\\noindent\n')
    f0.write('  \\begin{enumerate}\n')
    f1.write("""
\\exercice*
""")
    f1.write(_('Poser et effectuer les operations suivantes.\n'))
    f1.write('\\begin{multicols}{2}\\noindent\n')
    f1.write('  \\begin{enumerate}\n')
    for i in xrange(nb_exos):
        a = randrange(nb_exos - i)
        j = ordre_exos.pop(a)
        tex_exos[j](f0, f1)
    f0.write('  \\end{enumerate}\n')
    f0.write('\\end{multicols}\n')
    f1.write('  \\end{enumerate}\n')
    f1.write('\\end{multicols}\n')


def tex_calcul_mental(f0, f1):
    f0.write("""
\\exercice
""")
    f0.write(_('Effectuer sans calculatrice :\n'))
    f0.write('\\begin{multicols}{4}\\noindent\n')
    f0.write('  \\begin{enumerate}\n')
    f1.write("""
\\exercice*
""")
    f1.write(_('Effectuer sans calculatrice :\n'))
    f1.write('\\begin{multicols}{4}\\noindent\n')
    f1.write('  \\begin{enumerate}\n')
    operations.tex_calcul_mental(f0, f1)
    f0.write('  \\end{enumerate}\n')
    f0.write('\\end{multicols}\n')
    f1.write('\\end{enumerate}\n')
    f1.write('\\end{multicols}\n')


def tex_dix(f0, f1):
    f0.write("""
\\exercice
""")
    f0.write(_('Compl\\\'eter sans calculatrice :\n'))
    f0.write('\\begin{multicols}{2}\\noindent\n')
    f0.write('  \\begin{enumerate}\n')
    f1.write("""
\\exercice*
""")
    f1.write(_('Compl\\\'eter sans calculatrice :\n'))
    f1.write('\\begin{multicols}{2}\\noindent\n')
    f1.write('  \\begin{enumerate}\n')
    operations.tex_dix(f0, f1)
    f0.write('  \\end{enumerate}\n')
    f0.write('\\end{multicols}\n')
    f1.write('  \\end{enumerate}\n')
    f1.write('\\end{multicols}\n')


def tex_conversions(f0, f1):
    f0.write("""
\\exercice
""")
    f0.write(_('Effectuer les conversions suivantes :\n'))
    f0.write('\\begin{multicols}{3}\\noindent\n')
    f0.write('  \\begin{enumerate}\n')
    f1.write("""
\\exercice*
""")
    f1.write(_('Effectuer les conversions suivantes :\n'))
    f1.write('\\begin{multicols}{2}\\noindent\n')
    f1.write('  \\begin{enumerate}\n')
    decimaux.tex_units(f0, f1)
    f0.write('  \\end{enumerate}\n')
    f0.write('\\end{multicols}\n')
    f1.write('  \\end{enumerate}\n')
    f1.write('\\end{multicols}\n')


def tex_decimaux(f0, f1):
    f0.write("""
\\exercice
""")
    f1.write("""
\\exercice*
""")
    decimaux.tex_place_virgule(f0, f1)


def tex_frac(f0, f1):
    f0.write("""
\\exercice
""")
    f1.write("""
\\exercice*
""")
    f0.write(_("Completer :\n"))
    f0.write('\\begin{multicols}{3}\\noindent\n')
    f0.write('  \\begin{enumerate}\n')
    f1.write(_("Completer :\n"))
    f1.write('\\begin{multicols}{3}\\noindent\n')
    f1.write('  \\begin{enumerate}\n')
    decimaux.tex_frac(f0, f1)
    f0.write('  \\end{enumerate}\n')
    f0.write('\\end{multicols}\n')
    f1.write('  \\end{enumerate}\n')
    f1.write('\\end{multicols}\n')


def tex_decomposition(f0, f1):
    f0.write("""
\\exercice
""")
    f1.write("""
\\exercice*
""")
    f0.write(_("Completer avec un nombre decimal :\n"))
    f0.write('\\begin{multicols}{2}\\noindent\n')
    f0.write('  \\begin{enumerate}\n')
    f1.write(_("Completer avec un nombre decimal :\n"))
    f1.write('\\begin{multicols}{2}\\noindent\n')
    f1.write('  \\begin{enumerate}\n')
    decimaux.tex_dec(f0, f1)
    f0.write('  \\end{enumerate}\n')
    f0.write('\\end{multicols}\n')
    f1.write('  \\end{enumerate}\n')
    f1.write('\\end{multicols}\n')


def tex_classer(f0, f1):
    f0.write("""
\\exercice
""")
    f0.write('\\begin{enumerate}\n')
    f0.write('  \\item ')
    f1.write("""
\\exercice*
""")
    f1.write('\\begin{enumerate}\n')
    f1.write('  \\item ')
    decimaux.classer(f0, f1)
    f0.write('\n  \\item ')
    f1.write('\n  \\item ')
    decimaux.classer(f0, f1)
    f0.write('\\end{enumerate}\n')
    f1.write('\\end{enumerate}\n')


def tex_droites(f0, f1):
    f0.write("""
\\exercice
""")
    f1.write("""
\\exercice*
""")
    f0.write(_("Completer :\\par\n"))
    f0.write('\\begin{tabular}{|p{3cm}|p{5cm}|c|}\n')
    f0.write('  \\hline\n')
    f0.write(_('  \\hfill{} \\textbf{Nom} \\hfill{} & \\hfill{}\\textbf{Cat\\\'egorie}\\hfill{} & \\textbf{Figure} \\\\\n \\hline\n'))
    f1.write(_("Completer :\\par\n"))
    f1.write('\\begin{tabular}{|p{3cm}|p{5cm}|c|}\n')
    f1.write('  \\hline\n')
    f1.write(_('  \\hfill{} \\textbf{Nom} \\hfill{} & \\hfill{}\\textbf{Cat\\\'egorie}\\hfill{} & \\textbf{Figure} \\\\\n \\hline\n'))
    droites.tex_droites(f0, f1)
    f0.write('\\end{tabular}\n')
    f1.write('\\end{tabular}\n')


def tex_perpendiculaires(f0, f1):
    f0.write("""
\\exercice
""")
    f1.write("""
\\exercice*
""")
    f0.write(_("Realiser les figures suivantes :\\par\n"))
    f0.write('\\begin{multicols}{2}\n')
    f1.write(_("Realiser les figures suivantes :\\par\n"))
    f1.write('\\begin{multicols}{2}\n')
    droites.enonce_perp(f0, f1)
    f0.write('  \\columnbreak\n')
    f1.write('  \\columnbreak\n')
    droites.enonce_perp(f0, f1)
    f0.write('\\end{multicols}\n')
    f1.write('\\end{multicols}\n')


def tex_proprietes(f0, f1):
    f0.write("""
\\exercice
""")
    f1.write("""
\\exercice*
""")
    f0.write(_("""Completer le tableau suivant :\\par
Les droites en gras sont paralleles.\\par
"""))
    f1.write(_("""Completer le tableau suivant :\\par
Les droites en gras sont paralleles.\\par
"""))
    droites.enonce_prop(f0, f1)


def tex_divisibles(f0, f1):
    f0.write("""
\\exercice
""")
    f0.write(_('Cocher les bonnes reponses :\\par\n'))
    f1.write("""
\\exercice*
""")
    f1.write(_('Cocher les bonnes reponses :\\par\n'))
    quotients.tex_divisible(f0, f1)


def fractions_partage(f0, f1):
    f0.write("""
\\exercice
""")
    f0.write('\\begin{multicols}{2}\n')
    f0.write('  \\begin{enumerate}\n')
    f1.write("""
\\exercice*
""")
    f1.write('\\begin{multicols}{2}\n')
    f1.write('  \\begin{enumerate}\n')
    fractions.exo_fraction_partage(f0, f1)
    f0.write('  \\end{enumerate}\n')
    f0.write('\\end{multicols}\n')
    f1.write('  \\end{enumerate}\n')
    f1.write('\\end{multicols}\n')


def fractions_abscisses(f0, f1):
    f0.write("""
\\exercice
""")
    f1.write("""
\\exercice*
""")
    fractions.questions_fractions_abscisses(f0, f1)


def symetrie_quadrillage(f0, f1):
    f0.write("""
\\exercice
""")
    f1.write("""
\\exercice
""")
    f0.write(_("Construire la symetrique de chacune des figures par rapport \\`a la droite en\n"))
    f0.write(_("utilisant le quadrillage :\\par\n"))
    f0.write("\\psset{unit=.9cm}\n")
    f1.write(_("Construire la symetrique de chacune des figures par rapport \\`a la droite en\n"))
    f1.write(_("utilisant le quadrillage :\\par\n"))
    f1.write("\\psset{unit=.9cm}\n")
    symetrie.exo_quadrillage(f0, f1)


def mesures_angles(f0, f1):
    f0.write("""
\\exercice
""")
    f1.write("""
\\exercice*
""")
    f0.write(_('Nommer, mesurer et donner la nature de chacun des angles suivants :\\par \n'))
    f1.write(_('Nommer, mesurer et donner la nature de chacun des angles suivants :\\par \n'))
    angles.mesure_angles(f0, f1)


def main(lst, fiche_exos, fiche_corrige, entete):
    modules = (
        tex_calcul_mental,
        #tex_ecrit_nombre,
        tex_decimaux,
        tex_frac,
        tex_decomposition,
        tex_conversions,
        tex_operations,
        tex_dix,
        tex_classer,
        tex_droites,
        tex_perpendiculaires,
        tex_proprietes,
        tex_divisibles,
        fractions_partage,
        fractions_abscisses,
        symetrie_quadrillage,
        mesures_angles,
        )
    f0 = open(fiche_exos, 'w', -1)
    f1 = open(fiche_corrige, 'w', -1)
    if entete:
        tex_entete(f0)
        tex_entete(f1)
        f1.write('\\setlength{\\columnseprule}{0.5pt}\n')
        f0.write(_('\\chead{\\Large{\\textsc{Fiche de r\\\'evisions}}}\n'))
        f1.write(_("\\chead{\\Large{\\textsc{Corrige de la fiche de r\\'evisions}}}\n"))
        f0.write(_('\\rhead{\\textsl{\\footnotesize{Classe de 6\\ieme}}}\n'))
        f1.write(_('\\rhead{\\textsl{\\footnotesize{Classe de 6\\ieme}}}\n'))
    for j in xrange(len(modules)):
        for i in xrange(lst[j]):
            modules[j](f0, f1)
    if entete:
        f0.write('\\end{document}\n')
        f1.write('\\end{document}\n')
    f0.close()
    f1.close()


